@@ -46,6 +46,9 @@ host = "http://localhost:8069"
 username = "admin"
 pwd = "1"
 
+name_cache = {}
+
+class NotInAddonsPath(Exception): pass
 
 def exe(*params):
     config = get_settings()
@@ -545,47 +548,14 @@ def update_view_in_db(filepath, lineno):
                         exe("ir.ui.view", "write", view_ids, {'arch_db': arch})
 
 
+all_modules_cache = None
 class Modules(object):
 
     def __init__(self):
-        cache_file = self._get_cache_path()
-
-        if self.is_git_clean():
-            if not cache_file or not cache_file.exists():
-                modules = self._get_modules()
-                if cache_file:
-                    cache_file.write_bytes(pickle.dumps(modules))
-                self.modules = modules
-            else:
-                try:
-                    self.modules = pickle.loads(cache_file.read_bytes())
-                except:
-                    cache_file.unlink()
-                    modules = self._get_modules()
-        else:
-            self.modules = self._get_modules()
-
-    def _get_cache_path(self):
-        # 
-        from git import Repo
-        if not (Path(os.getcwd()) / '.git').exists():
-            return None
-        repo = Repo(os.getcwd())
-        sha = repo.head.commit.hexsha
-        full_path = os.getcwd().replace('/', '_')
-        if os.getenv("SUDO_UID"):
-            uid = int(os.getenv("SUDO_UID"))
-        else:
-            uid = os.getuid()
-        parent = Path(f"/tmp/.odoo.modules.{uid}.{full_path}")
-        parent.mkdir(exist_ok=True)
-        if os.getenv("SUDO_USER"):
-            try_to_set_owner(
-                int(os.environ['SUDO_UID']),
-                parent,
-            )
-        
-        return parent / sha
+        global all_modules_cache
+        if not all_modules_cache:
+            all_modules_cache = self._get_modules()
+        self.modules = all_modules_cache
 
     def _get_modules(self):
         modnames = set()
@@ -645,8 +615,8 @@ class Modules(object):
 
         - fetches to be installed modules from install-file
         - selects all installed, to_be_installed, to_upgrade modules from db and checks wether
-          they are from "us" or OCA
-          (often those modules are forgotten to be updated)
+            they are from "us" or OCA
+            (often those modules are forgotten to be updated)
 
         """
         assert mode in [None, 'to_update', 'to_install']
@@ -656,6 +626,7 @@ class Modules(object):
         if mode == 'to_install':
             modules = [x for x in modules if not DBModules.is_module_installed(x)]
 
+        modules = list(map(lambda x: Module.get_by_name(x), modules))
         return modules
 
     def get_module_dependency_tree(self, module):
@@ -680,7 +651,10 @@ class Modules(object):
                 try:
                     dep_mod = dep_mod[0]
                 except Exception:
-                    click.secho(f"Module not found: {dep}\n\n\n{list(sorted(map(lambda x: x.name, self.modules.values())))}", fg='red', bold=True)
+                    click.secho((
+                        f"Module not found: {dep}\n\n\n"
+                        f"{list(sorted(map(lambda x: x.name, self.modules.values())))}"
+                    ), fg='red', bold=True)
                     sys.exit(-1)
                 data[mod.name][dep] = {}
                 append_deps(dep_mod, data[mod.name][dep])
@@ -702,48 +676,75 @@ class Modules(object):
 
         x(deptree)
         assert all(isinstance(x, str) for x in result)
+        result = list(map(lambda x: Module.get_by_name(x), list(result)))
         return sorted(list(result))
 
+    def get_all_auto_install_modules(self):
+        auto_install_modules = []
+        for module in Modules()._get_modules():
+            try:
+                module = Module.get_by_name(module)
+            except NotInAddonsPath:
+                continue
+            if module.manifest_dict.get('auto_install', False):
+                auto_install_modules.append(module)
+        return auto_install_modules
+
+    def get_filtered_auto_install_modules_based_on_module_list(self, module_list):
+        module_list = list(map(lambda x: Module.get_by_name(x), module_list))
+
+        complete_modules = set()
+        for mod in module_list:
+            complete_modules |= set(list(self.get_module_flat_dependency_tree(mod)))
+
+        def _get(all_modules):
+            for auto_install_module in all_modules:
+                dependencies = set(list(
+                    self.get_module_flat_dependency_tree(auto_install_module)))
+                installed_dependencies = set([
+                    x for x in dependencies
+                    if x.manifest_dict.get('auto_install') or x in complete_modules
+                ])
+                if dependencies == installed_dependencies:
+                    yield auto_install_module
+
+                    if all(x in module_list for x in dependencies):
+                        yield auto_install_module
+
+        modules = self.get_all_auto_install_modules()
+        while True:
+            before = list(sorted(set(map(lambda x: x.name, modules))))
+            modules = list(_get(modules))
+            after = list(sorted(set(map(lambda x: x.name, modules))))
+            if after == before:
+                break
+        return modules
+
     def get_all_used_modules(self):
         """
         Returns all modules that are directly or indirectly (auto install, depends) installed.
         """
         result = set()
         modules = self.get_customs_modules()
+        auto_install_modules = self.get_filtered_auto_install_modules_based_on_module_list(modules)
+        modules += auto_install_modules
 
-        auto_install_modules = []
         for module in modules:
-            module = Module.get_by_name(module)
             result.add(module.name)
-            if module.manifest_dict.get('auto_install', False):
-                auto_install_modules.append(module)
             dependencies = self.get_module_flat_dependency_tree(module)
             for dep in dependencies:
                 result.add(dep)
 
-        # check for auto install modules - auto install could refer to other auto install
-        while True:
-            changed = False
-            for module in list(auto_install_modules):
-                depends = [x for x in module.manifest_dict.get('depends', []) if x != 'base']
-                if all(x in modules for x in depends) and module.name not in result:
-                    changed = True
-                    result.append(module.name)
-                    auto_install_modules.remove(module)
-            if not changed:
-                break
-
         return list(result)
 
-    def get_all_external_dependencies(self, additional_modules=[]):
+    def get_all_external_dependencies(self, additional_modules=None):
+        additional_modules = additional_modules or []
         modules = self.get_all_used_modules()
         modules = list(sorted(set(modules) | set(additional_modules)))
         pydeps = []
         deb_deps = []
         for module in modules:
-            if module == 'master':
-                import pudb;pudb.set_trace()
-            module = self.modules[module]
+            module = Module.get_by_name(module)
             file = (module.path / 'external_dependencies.txt')
             new_deps = []
             if file.exists():
@@ -838,8 +839,29 @@ class Module(object):
 
     class IsNot(Exception): pass
 
+    def __str__(self):
+        return (
+            f"{self.name}"
+        )
+
+    def __repr__(self):
+        return (
+            f"{self.name}"
+        )
+
+    def __lt__(self, other):
+        if isinstance(other, str):
+            return self.name < other
+        return self.name < other.name
+
+    def __gt__(self, other):
+        if isinstance(other, str):
+            return self.name > other
+        return self.name > other.name
+
     def __init__(self, path):
         self.version = float(current_version())
+        self._manifest_dict = None
         path = Path(path)
         p = path if path.is_dir() else path.parent
 
@@ -860,16 +882,20 @@ class Module(object):
 
     @property
     def manifest_dict(self):
-        try:
-            content = self.manifest_path.read_text()
-            content = '\n'.join(filter(lambda x: not x.strip().startswith("#"), content.split("\n")))
-            return eval(content) # TODO safe
-        except SyntaxError:
-            click.secho("error at file: %s" % self.manifest_path, fg='red')
-            raise
-        except Exception:
-            click.secho("error at file: %s" % self.manifest_path, fg='red')
-            raise
+        if not self._manifest_dict:
+            try:
+                content = self.manifest_path.read_text()
+                content = '\n'.join(filter(lambda x: not x.strip().startswith("#"), content.split("\n")))
+                return eval(content) # TODO safe
+            except SyntaxError:
+                click.secho("error at file: %s" % self.manifest_path, fg='red')
+                raise
+            except Exception:
+                click.secho("error at file: %s" % self.manifest_path, fg='red')
+                raise
+            else:
+                self._manifest_dict = content
+        return self._manifest_dict
 
     def __make_path_relative(self, path):
         path = path.resolve().absolute()
@@ -883,15 +909,35 @@ class Module(object):
         pofile_path - pathin in the machine
         """
         pofile_path = self.__make_path_relative(pofile_path)
-        LANG = pofile_path.name.split(".po")[0]
-        write_debug_instruction('import_i18n:{}:{}'.format(LANG, pofile_path))
+        lang = pofile_path.name.split(".po")[0]
+        write_debug_instruction('import_i18n:{}:{}'.format(lang, pofile_path))
 
-    def export_lang(self, current_file, LANG):
-        write_debug_instruction('export_i18n:{}:{}'.format(LANG, self.name))
+    def export_lang(self, current_file, lang):
+        write_debug_instruction('export_i18n:{}:{}'.format(lang, self.name))
+
+    @property
+    def hash(self):
+        from .tools import get_directory_hash
+        return get_directory_hash(self.path)
 
     @classmethod
-    def get_by_name(clazz, name):
+    def __get_by_name_cached(cls, name):
+        if name not in name_cache:
+            name_cache.setdefault(
+                name, cls._get_by_name(name))
+        return name_cache[name]
+
+    @classmethod
+    def get_by_name(cls, name):
+        if isinstance(name, Module):
+            return name
+        return cls.__get_by_name_cached(name)
+
+    @classmethod
+    def _get_by_name(cls, name):
         from .odoo_config import get_odoo_addons_paths
+        if isinstance(name, Module):
+            name = name.name
         path = None
         for addon_path in get_odoo_addons_paths():
             dir = addon_path / name
@@ -899,7 +945,7 @@ class Module(object):
                 path = dir
             del dir
         if not path:
-            raise Exception("Could not get path for {}".format(name))
+            raise NotInAddonsPath(f"Could not get path for {name}")
         if path.exists():
             path = path.resolve()
 
@@ -912,7 +958,6 @@ class Module(object):
 
         # could be an odoo module then
         for path in get_odoo_addons_paths():
-            print(path)
             if (path / name).resolve().is_dir():
                 return Module(path / name)
         raise Exception("Module not found or not linked: {}".format(name))
